/** \file pyramid.hh
    \brief File that will manage the combinatorial pyramid */

#ifndef __PYRAMID_HH__
#define __PYRAMID_HH__

#include <tab_bool.hh>
#include <image.hh>

#include "value_set.hh"
#include "combinatorial_map.hh"

class combi_map_base;
class combi_map_base_exp;
class combi_map_base_imp;
class kernel;


/** Class that will manage the combinatorial pyramid */
class pyramid
{
private:
  // Pointor on the combinatorial map of the base of the pyramid
  combi_map_base *base;
  // The combinatorial map of the top level of the pyramid
  combi_map_priv map;
  // The level_set of the pyramid
  level_set l_set;
  // Pointor on the value_set of the pyramid
  value_set *value;
  
public:
  /// Enum used to determine if the map of the base must be implicit or explicit
  typedef enum {implicit_base,explicit_base} construction;
  /// Constructor
  pyramid(int dimx, int dimy, value_set &val, construction c=explicit_base);
  /// Constructor
  pyramid(char *, construction c=explicit_base);
  /// Constructor
  pyramid(ifstream &, construction c=explicit_base);
  /// Destructor
  ~pyramid();
  /// Fonction to generate one level of the pyramid
  void compute_level(kernel &);
  /// Fonction to generate all the level of the pyramid
  void compute_pyramid(kernel &);
  /// Save pyramid into a flow
  int save(const char*);
  /// Compute the combinatorial map of one level of the pyramid
  void give_combinatorial_map(int,combinatorial_map &);
  /// Give the maximum level of the pyramid
  int give_max_level();
  
  /// Give the alpha of the dart on the base map
  dart give_alpha_base(dart d)
  {
    return base->alpha(d);
  }
  /// Give the sigma of the dart on the base map
  dart give_sigma_base(dart d)
  {
    return base->sigma(d);
  }
  /// Give the move of the dart on the base map
  move_type give_move_base(dart d)
  {
    return base->give_move(d);
  }
  /// Give the max level of the dart d
  int give_level(dart d)
  {
    return l_set.give_level(d);
  }
  /// Give the departure point of the dart
  void give_pointel(dart d,point &p)
  {
    base->give_pointel(d,p);
  }
  /// Give the initial pixel of the vertex describe by the dart in the base map
  void give_pixel(dart d,point &p)
  {
    base->give_pixel(d,p);
  }

  /// Function that compute the variation of orientation between two darts orientation
  int compute_index_orientation(dart d1,dart d2)
  {
    return base->compute_index_orientation(d1, d2);
  }
  /// Give an iterator positionned on the first dart of the level_set for the given level
  dart_iterator &first_dart(int l)
  {
    return l_set.first(l);
  }
  /// Give an iterator positionned on the first dart of the given level
  dart_iterator &begin_dart(int l)
  {
    return l_set.begin(l);
  }
  /// Give an iterator positionned after the last dart of the given level (loop : for(i=level_set.first();i<level.end();i++)
  dart_iterator &end_dart(int l)
  {
    return l_set.end(l);
  }
  /// Give an iterator positionned on the last dart of the given level (loop : for(i=level_set.first();i<=level.last();i++)
  dart_iterator &last_dart(int l)
  {
    return l_set.last(l);
  }
  /// Give the width of the base of the pyramid
  int give_width()
  {
    return base->give_width();
  }
  /// Give the height of the base of the pyramid
  int give_height()
  {
    return base->give_height();
  }

  /// Fonction to compute the next dart of the border for the given dart of the given level;
  dart give_next_border_dart(dart,int);

private:
  /// Set the pyramid from a flow
  int load(ifstream &, construction);
  // Fonction that will be call to contract the edge marked by the kernel
  void contract_edge(kernel &, list_label &);
  // Fonction that will be call to remove the empty self-loop
  void remove_empty_self_loops(list_label &);
  // Fonction that will be call to remove double edge
  void remove_double_edges(kernel &);
  // Fonction to update the mu and mu_1 array, and the vertex list of the combinatorial map 
  void update_vertex_set(kernel &);
  /// Function to updates the sigma successor of a surviving dart during the removal of double edges.
  void update_sigma_double_edge(kernel &,list_label &,tab_sym_bool &,dart);
  /// Function that compute and initialize if needed the sigma successor
  dart compute_sigma_successor(kernel &kern, dart b);
  /// Function that compute and initialize the next sigma and the alpha of a given dart
  dart update_sigma_alpha(dart, dart &, int, dart &);
  /// Function that compute and initialize the next dart of a given dart
  dart update_next_dart(dart, dart &, int, bool &);

  /// Function that compute and initialize the next sigma and the alpha of a given dart a given level of the pyramid
  dart extract_update_sigma_alpha(dart,dart&,int,int&,tab_dart&);
  /// Function that compute and initialize the next dart of a given dart a given level of the pyramid
  dart extract_update_next_dart(dart,dart&,int,tab_dart&);
};


#endif // __PYRAMID_HH__
